/*  $Id: graphic_context.h,v 1.30 2003/10/14 15:52:10 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

//! clanDisplay="Display 2D"
//! header=display.h

#ifndef header_graphic_context
#define header_graphic_context

#if _MSC_VER > 1000
#pragma once
#endif

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include "../signals.h"
#include <stack>
#include "color.h"
#include "gradient.h"
#include "../Core/Math/point.h"
#include "../Core/Math/rect.h"

#ifndef GL_CLAMP_TO_EDGE_EXT
#define GL_CLAMP_TO_EDGE_EXT 0x812F
#endif

class CL_GraphicContext_Generic;

//: Graphic rendering target.
//- <p>A graphic context is something that ClanLib can render onto.</p>
class CL_GraphicContext
{
//! Construction:
public:
	//: Constructs a graphic context.
	CL_GraphicContext();

	CL_GraphicContext(const CL_GraphicContext &copy);

	virtual ~CL_GraphicContext();

//! Attributes:
public:
	//: Returns the current width of the context.
	int get_width() const;

	//: Returns the current height of the context.
	int get_height() const;

	//: Returns the current clipping rectangle used on the graphic context.
	const CL_Rect &get_cliprect() const;

	//: Returns the current effective x-axis translation offset.
	int get_translate_x() const;
	
	//: Returns the current effective y-axis translation offset.
	int get_translate_y() const;

	//: Returns a pixel value at specified coordinates.
	CL_Color get_pixel(int x, int y) const;

//! Operations:
public:
	//: Copy assignment operator.
	CL_GraphicContext &operator =(const CL_GraphicContext &copy);

	//: Begin window 3D rendering mode.
	void begin_3d();

	//: End windows 3D rendering mode.
	void end_3d();

	//: Flushes current rendering batch.
	//- <p>With the OpenGL target, this causes the graphic context to end its current
	//- active glBegin()/glEnd() pair, making it possible to call other OpenGL calls.</p>
	void flush();

	//: Draw a pixel at (x, y) using the specified color.
	void draw_pixel(int x, int y, const CL_Color &color);

	//: Draw a line from (x1, y1) to (x2, y2) using the specified color.
	void draw_line(int x1, int y1, int x2, int y2, const CL_Color &color);

	//: Draw a rectangle using the specified color.
	void draw_rect(const CL_Rect &rect, const CL_Color &color);

	//: Draw a gradient rectangle using the specified gradient.
	void fill_rect(const CL_Rect &rect, const CL_Gradient &gradient);

	//: Draw a filled rectangle using the specified color.
	void fill_rect(const CL_Rect &rect, const CL_Color &color);

	//: Clears the whole context using the specified color.
	void clear(const CL_Color &color = CL_Color(0,0,0));

	//: Set the current clipping rectangle.
	void set_cliprect(const CL_Rect &rect);

	//: Push current clipping rectangle to stack.
	//- <p>If a rectangle is passed, it afterwards sets clipping
	//- rectangle to the union of the current rectangle and the passed
	//- rectangle.</p>
	void push_cliprect(const CL_Rect &rect);

	void push_cliprect();

	//: Pop current clipping rectangle from the stack.
	void pop_cliprect();

	//: Sets the translate offset.
	//- <p> The new offset will disregard any previous offset's, but will not
	//- empty the translation stack. The new translation offset will affect
	//- any subsequent display operations on the current displaycard, by
	//- translating the position of the display operation with the offset. </p>
	void set_translate_offset(int x, int y);
	
	//: Push translation offset onto translation stack.
	//- <p> This offset will affect any subsequent display operations on the current
	//- displaycard, by translating the position of the display operation with the offset.
	//- The offset will be offset by any previous offsets pushed onto the stack,
	//- eg. it inherits the previous offset. </p>
	void push_translate_offset(int delta_x, int delta_y);
	
	void push_translate_offset();
	
	//: Pops the last pushed translation offset from the translation offset stack.
	//- <p> If the stack is empty, nothing will happen, and if the last
	//- translation offset is popped, the translation offset will be set to 0,0. </p>
	void pop_translate_offset();

//! Implementation:
public:
	CL_GraphicContext(CL_GraphicContext_Generic *impl);
	
	CL_GraphicContext_Generic *impl;
};

#endif
